import ocpa.algo
import ocpa.objects
import ocpa.util
import ocpa.visualization

__version__ = '0.0.10'
__doc__ = "Object-Centric Process Analysis"
__author__ = 'Gyunam Park and Niklas Adams'
__author_email__ = 'gnpark@pads.rwth-aachen.de'
__maintainer__ = 'Gyunam Park and Niklas Adams'
__maintainer_email__ = "gnpark@pads.rwth-aachen.de"
